@@ -8,23 +8,7 @@
  ******************************************************************************
  * Copyright (c) 2002, Frank Warmerdam <warmerdam@pobox.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #ifndef OGR_OCI_H_INCLUDED
@@ -34,6 +18,9 @@
 #include "oci.h"
 #include "cpl_error.h"
 
+#include <map>
+#include <set>
+
 /* -------------------------------------------------------------------- */
 /*      Low level Oracle spatial declarations.                          */
 /* -------------------------------------------------------------------- */
@@ -150,6 +137,7 @@ class CPL_DLL OGROCIStatement
     {
         return hStatement;
     }
+
     CPLErr BindScalar(const char *pszPlaceName, void *pData, int nDataLen,
                       int nSQLType, sb2 *paeInd = nullptr);
     CPLErr BindString(const char *pszPlaceName, const char *pszData,
@@ -212,11 +200,13 @@ class OGROCIStringBuf
     char *StealString();
 
     char GetLast();
+
     char *GetEnd()
     {
         UpdateEnd();
         return pszString + nLen;
     }
+
     char *GetString()
     {
         return pszString;
@@ -234,12 +224,12 @@ class OGROCIDataSource;
 class OGROCILayer CPL_NON_FINAL : public OGRLayer
 {
   protected:
+    OGROCIDataSource *poDS;
+
     OGRFeatureDefn *poFeatureDefn;
 
     int iNextShapeId;
 
-    OGROCIDataSource *poDS;
-
     char *pszQueryStatement;
 
     int nResultOffset;
@@ -257,6 +247,8 @@ class OGROCILayer CPL_NON_FINAL : public OGRLayer
     char *pszFIDName;
     int iFIDColumn;
 
+    std::set<int> setFieldIndexWithTimeStampWithTZ{};
+
     OGRGeometry *TranslateGeometry();
     OGRGeometry *TranslateGeometryElement(int *piElement, int nGType,
                                           int nDimension, int nEType,
@@ -269,8 +261,9 @@ class OGROCILayer CPL_NON_FINAL : public OGRLayer
                         double *pdfY, double *pdfZ);
 
   public:
-    OGROCILayer();
+    explicit OGROCILayer(OGROCIDataSource *poDSIn);
     virtual ~OGROCILayer();
+
     virtual int FindFieldIndex(const char *pszFieldName,
                                int bExactMatch) override
     {
@@ -291,6 +284,8 @@ class OGROCILayer CPL_NON_FINAL : public OGRLayer
     virtual const char *GetFIDColumn() override;
     virtual const char *GetGeometryColumn() override;
 
+    GDALDataset *GetDataset() override;
+
     int LookupTableSRID();
 };
 
@@ -331,7 +326,7 @@ class OGROCIWritableLayer CPL_NON_FINAL : public OGROCILayer
 
     void ParseDIMINFO(const char *, double *, double *, double *);
 
-    OGROCIWritableLayer();
+    explicit OGROCIWritableLayer(OGROCIDataSource *poDSIn);
     virtual ~OGROCIWritableLayer();
 
   public:
@@ -339,23 +334,27 @@ class OGROCIWritableLayer CPL_NON_FINAL : public OGROCILayer
     {
         return poSRS;
     }
-    virtual OGRErr CreateField(OGRFieldDefn *poField,
+
+    virtual OGRErr CreateField(const OGRFieldDefn *poField,
                                int bApproxOK = TRUE) override;
     virtual int FindFieldIndex(const char *pszFieldName,
                                int bExactMatch) override;
 
     // following methods are not base class overrides
-    void SetOptions(char **);
+    void SetOptions(CSLConstList);
 
     void SetDimension(int);
+
     void SetLaunderFlag(int bFlag)
     {
         bLaunderColumnNames = bFlag;
     }
+
     void SetPrecisionFlag(int bFlag)
     {
         bPreservePrecision = bFlag;
     }
+
     void SetDefaultStringSize(int nSize)
     {
         nDefaultStringSize = nSize;
@@ -405,6 +404,7 @@ class OGROCILoaderLayer final : public OGROCIWritableLayer
     virtual void SetSpatialFilter(OGRGeometry *) override
     {
     }
+
     virtual void SetSpatialFilter(int iGeomField, OGRGeometry *poGeom) override
     {
         OGRLayer::SetSpatialFilter(iGeomField, poGeom);
@@ -497,6 +497,7 @@ class OGROCITableLayer final : public OGROCIWritableLayer
     virtual GIntBig GetFeatureCount(int) override;
 
     virtual void SetSpatialFilter(OGRGeometry *) override;
+
     virtual void SetSpatialFilter(int iGeomField, OGRGeometry *poGeom) override
     {
         OGRLayer::SetSpatialFilter(iGeomField, poGeom);
@@ -512,6 +513,7 @@ class OGROCITableLayer final : public OGROCIWritableLayer
     virtual OGRErr DeleteFeature(GIntBig nFID) override;
 
     virtual OGRErr GetExtent(OGREnvelope *psExtent, int bForce = TRUE) override;
+
     virtual OGRErr GetExtent(int iGeomField, OGREnvelope *psExtent,
                              int bForce) override
     {
@@ -549,12 +551,11 @@ class OGROCISelectLayer final : public OGROCILayer
 /*                           OGROCIDataSource                           */
 /************************************************************************/
 
-class OGROCIDataSource final : public OGRDataSource
+class OGROCIDataSource final : public GDALDataset
 {
     OGROCILayer **papoLayers;
     int nLayers;
 
-    char *pszName;
     char *pszDBName;
 
     int bDSUpdate;
@@ -564,9 +565,9 @@ class OGROCIDataSource final : public OGRDataSource
 
     // We maintain a list of known SRID to reduce the number of trips to
     // the database to get SRSes.
-    int nKnownSRID;
-    int *panSRID;
-    OGRSpatialReference **papoSRS;
+    std::map<int,
+             std::unique_ptr<OGRSpatialReference, OGRSpatialReferenceReleaser>>
+        m_oSRSCache{};
 
   public:
     OGROCIDataSource();
@@ -582,22 +583,19 @@ class OGROCIDataSource final : public OGRDataSource
     int OpenTable(const char *pszTableName, int nSRID, int bUpdate,
                   int bTestOpen, char **papszOpenOptionsIn);
 
-    const char *GetName() override
-    {
-        return pszName;
-    }
     int GetLayerCount() override
     {
         return nLayers;
     }
+
     OGRLayer *GetLayer(int) override;
     OGRLayer *GetLayerByName(const char *pszName) override;
 
     virtual OGRErr DeleteLayer(int) override;
-    virtual OGRLayer *ICreateLayer(const char *,
-                                   OGRSpatialReference * = nullptr,
-                                   OGRwkbGeometryType = wkbUnknown,
-                                   char ** = nullptr) override;
+
+    OGRLayer *ICreateLayer(const char *pszName,
+                           const OGRGeomFieldDefn *poGeomFieldDefn,
+                           CSLConstList papszOptions) override;
 
     int TestCapability(const char *) override;
 
@@ -611,7 +609,7 @@ class OGROCIDataSource final : public OGRDataSource
                                  const char *pszDialect) override;
     virtual void ReleaseResultSet(OGRLayer *poLayer) override;
 
-    int FetchSRSId(OGRSpatialReference *poSRS);
+    int FetchSRSId(const OGRSpatialReference *poSRS);
     OGRSpatialReference *FetchSRS(int nSRID);
 };
 
